from __future__ import print_function, unicode_literals

import re
import os
import six
import copy

from preup.xml_manager import html_escape_string
from preup.utils import get_assessment_version, get_file_content, write_to_file
from preup.utils import print_error_msg
from preup import settings
from preuputils import xml_tags
from preuputils import script_utils
from preup.exception import MissingTagsIniFileError, EmptyTagIniFileError


def get_full_xml_tag(dirname):
    """We need just from RHEL directory"""
    found = 0
    # Get index of scenario and cut directory till scenario (included)
    for index, dir_name in enumerate(dirname.split(os.path.sep)):
        if re.match(r'\D+(\d)_(\D*)(\d)-results', dir_name, re.I):
            found = index
    main_dir = dirname.split(os.path.sep)[found+1:]
    return main_dir


class XmlUtils(object):
    """Class generate a XML from xml_tags and loaded INI file"""
    def __init__(self, dir_name, ini_files):
        self.keys = {}
        self.select_rules = []
        self.rule = []
        self.dirname = dir_name
        self.ini_files = ini_files
        self._test_init_file()

    def _test_init_file(self):
        test_dict = copy.deepcopy(self.ini_files)
        allowed_tags = ['check_script', 'content_description', 'content_title', 'applies_to',
                        'author', 'binary_req', 'solution', 'bugzilla', 'config_file',
                        'group_title', 'mode', 'requires', 'solution_type']
        for ini, content in six.iteritems(test_dict):
            content_dict = content[0]
            for tag in allowed_tags:
                if tag in content_dict:
                    del content_dict[tag]
            if content_dict:
                tags = ','. join(six.iterkeys(content_dict))
                print_error_msg(title="The tag '%s' is not allowed in INI file %s." % (tags, ini),
                                msg="\nAllowed tags for contents are %s" % ','.join(allowed_tags),
                                level=' WARNING ')

    def update_files(self, file_name, content):
        """Function updates file_name <migrate or update> according to INI file."""

        """
        :param file_name: specified in INI file like mode: upgrade, migrate
        :param content: name of the content like xccdf_rule_...
        :return: Nothing
        """
        path_name = os.path.join(settings.UPGRADE_PATH, file_name)
        lines = []
        if os.path.exists(path_name):
            lines = get_file_content(path_name, 'rb', method=True)
        test_content = [x.strip() for x in lines if content in x.strip()]
        if not test_content:
            lines.append(content + '\n')
            write_to_file(path_name, 'wb', lines)

    def _update_check_description(self, filename):
        new_text = []
        lines = get_file_content(os.patch.join(self.dirname, filename), "rb", True)

        bold = '<xhtml:b>{0}</xhtml:b>'
        br = '<xhtml:br/>'
        table_begin = '<xhtml:table>'
        table_end = '</xhtml:table>'
        table_header = '<xhtml:tr><xhtml:th>Result</xhtml:th><xhtml:th>Description</xhtml:th></xhtml:tr>'
        table_row = '<xhtml:tr><xhtml:td>{0}</xhtml:td><xhtml:td>{1}</xhtml:td></xhtml:tr>'
        new_text.append(br + br + '\n' + bold.format('Details:') + br)
        results = False
        for index, line in enumerate(lines):
            if '=' in line:
                if not results:
                    new_text.append(bold.format('Expected results:') + br)
                    new_text.append(table_begin + '\n' + table_header)
                    results = True
                try:
                    exp_results = line.strip().split('=')
                    new_text.append(table_row.format(exp_results[0], exp_results[1]) + '\n')
                except IndexError:
                    pass
            else:
                new_text.append(line.rstrip() + br)
        if results:
            new_text.append(table_end + '\n')

        return '\n'.join(new_text)

    def update_values_list(self, section, search_exp, replace_exp):
        """
        The function replaces tags taken from INI files.
        Tags are mentioned in xml_tags.py
        """
        forbidden_empty = ["{scap_name}", "{main_dir}"]
        if search_exp == "{content_description}":
            replace_exp = replace_exp.rstrip()
        elif search_exp == "{check_description}":
            replace_exp = '\n' + replace_exp + '\n'
        elif search_exp == "{config_file}":
            new_text = ""
            for lines in replace_exp.split(','):
                new_text = new_text+"<xhtml:li>"+lines.strip()+"</xhtml:li>"
            replace_exp = new_text.rstrip()
        elif search_exp == "{solution}":
            new_text = get_file_content(os.path.join(self.dirname, replace_exp), "rb", True)
            # we does not need interpreter for fix script
            # in XML therefore skip first line
            replace_exp = ''.join(new_text[1:])
        elif search_exp == "{solution_text}":
            new_text = "_" + '_'.join(get_full_xml_tag(self.dirname))\
                       + "_SOLUTION_MSG_" + replace_exp.upper()
            replace_exp = new_text
        if replace_exp == '' and search_exp in forbidden_empty:
            print_error_msg(title="Disapproved empty replacement for tag '%s'" % search_exp)
            raise EmptyTagIniFileError

        for cnt, line in enumerate(section):
            if search_exp in line:
                section[cnt] = line.replace(search_exp, replace_exp)

    def check_recommended_fields(self, keys=None, script_name=""):
        """
        The function checks whether all fields in INI file are fullfiled
        If solution_type is mentioned than HTML page can be used.
        HTML solution type can contain standard HTML tags

        field are needed by YAML file
        """
        fields = ['content_title', 'check_script', 'solution', 'applies_to']
        optional = ['solution_type']
        unused = [x for x in fields if not keys.get(x)]
        if unused:
            title = 'Following tags are missing in INI file %s\n' % script_name
            if 'applies_to' not in unused:
                print_error_msg(title=title, msg=' '.join(unused))
                raise MissingTagsIniFileError
        if 'solution_type' in keys:
            if keys.get('solution_type') == "html" or keys.get('solution_type') == "text":
                pass
            else:
                print_error_msg(
                    title="Wrong solution_type. Allowed are 'html' or 'text' %s" % script_name
                )
                os.sys.exit(0)

    def add_value_tag(self):
        """The function adds VALUE tag in group.xml file"""
        value_tag = []
        check_export_tag = list()
        check_export_tag.append(xml_tags.RULE_SECTION_VALUE_IMPORT)
        for key, val in xml_tags.DIC_VALUES.items():
            value_tag.append(xml_tags.VALUE)
            if key == 'current_directory':
                val = '/'.join(get_full_xml_tag(self.dirname))
                val = 'SCENARIO/' + val
            self.update_values_list(value_tag, "{value_name}", val)
            self.update_values_list(value_tag, "{val}", key.lower())
            check_export_tag.append(xml_tags.RULE_SECTION_VALUE)
            self.update_values_list(check_export_tag, "{value_name_upper}", key.upper())
            self.update_values_list(check_export_tag, "{val}", key.lower())
        for key, val in xml_tags.GLOBAL_DIC_VALUES.items():
            check_export_tag.append(xml_tags.RULE_SECTION_VALUE_GLOBAL)
            self.update_values_list(check_export_tag, "{value_name_upper}", key.upper())
            self.update_values_list(check_export_tag, "{value_name}", key)

        return value_tag, check_export_tag

    def solution_modification(self, key):
        """Function handles a solution text or scripts"""
        fix_tag = []
        for k in key['solution'].split(','):
            k = k.strip()
            script_utils.check_scripts('solution', self.dirname, script_name=k)
            script_type = script_utils.get_script_type(self.dirname, script_name=k)
            if script_type == "txt":
                fix_tag.append(xml_tags.FIX_TEXT)
            else:
                fix_tag.append(xml_tags.FIX)

            self.update_values_list(fix_tag, "{solution_text}",
                                    key['solution_type'] if 'solution_type' in key else "text")
            self.update_values_list(fix_tag, "{solution}", k)
            self.update_values_list(fix_tag, "{script_type}", script_type)
        self.update_values_list(self.rule, '{fix}', ''.join(fix_tag))

    def check_script_modification(self, key, k):
        """Function checks a check script"""
        script_utils.check_scripts(k, self.dirname, script_name=key[k])
        check_func = {'log_': ['log_none_risk', 'log_slight_risk',
                               'log_medium_risk', 'log_high_risk',
                               'log_extreme_risk', 'log_info',
                               'log_error', 'log_warning'],
                      'exit_': ['exit_error', 'exit_fail',
                                'exit_fixed', 'exit_not_applicable',
                                'exit_pass', 'exit_unknown',
                                'exit_informational']}
        for check in check_func:
            script_utils.check_inplace_risk(self.dirname,
                                            prefix=check,
                                            script_name=key[k],
                                            check_func=check_func[check])
        self.update_values_list(self.rule, "{scap_name}", key[k].split('.')[0])
        requirements = {'applies_to': 'check_applies',
                        'binary_req': 'check_bin',
                        'requires': 'check_rpm'}
        updates = dict()
        for req in requirements:
            if req in key:
                updates[requirements[req]] = key[req]
        script_utils.update_check_script(self.dirname,
                                         updates,
                                         script_name=key[k],
                                         author=key['author'] if 'author' in key else "")
        self.update_values_list(self.rule, "{"+k+"}", key[k])

    def prepare_sections(self):
        """The function prepares all tags needed for generation group.xml file."""
        group_ini = False
        for main, self.keys in self.ini_files.items():
            if main.endswith("group.ini"):
                self.rule.append(xml_tags.GROUP_INI)
                for k in self.keys:
                    self.update_values_list(self.rule, '{group_title}', k['group_title'])
                    self.update_values_list(self.rule, '{group_value}', "")
            else:
                self.rule.append(xml_tags.CONTENT_INI)
                self.create_xml_from_ini(main)
                self.update_values_list(self.rule,
                                        "{select_rules}",
                                        ' '.join(self.select_rules))
            xml_tag = "{main_dir}"
            self.update_values_list(self.rule,
                                    xml_tag,
                                    '_'.join(get_full_xml_tag(self.dirname)))
        return self.rule

    def fnc_config_file(self, key, name):
        """Function updates a config file."""
        if name in key and key[name] is not None:
            self.update_values_list(self.rule, "{config_section}",
                                    xml_tags.CONFIG_SECTION)
            self.update_values_list(self.rule, "{config_file}",
                                    key['config_file'])
        else:
            self.update_values_list(self.rule, "{config_section}", "")

    def fnc_check_script(self, key, name):
        """ Function updates a check_script """
        if name in key:
            self.check_script_modification(key, name)
            self.update_values_list(self.select_rules, "{scap_name}",
                                    key[name].split('.')[0])

    def fnc_check_description(self, key, name):
        """ Function updates a check_description """
        if name in key and key[name] is not None:
            escaped_text = self._update_check_description(key[name])
            self.update_values_list(self.rule, "{check_description}", escaped_text)
        else:
            self.update_values_list(self.rule, "{check_description}", "")

    def fnc_solution_text(self, key, name):
        """Function updates a solution text."""
        if name in key:
            self.solution_modification(key)
        else:
            self.update_values_list(self.rule, "{fix}", xml_tags.FIX_TEXT)
            self.update_values_list(self.rule, "{solution_text}", "text")
            self.update_values_list(self.rule, "{platform_id}",
                                    get_assessment_version(self.dirname)[1])

    def fnc_update_mode(self, key, name):

        """
        Function update <upgrade_path>/<migrate.conf|update.conf> files
        migrate_xccdf_path
        :param key:
        :param name:
        :return:
        """

        content = "{rule}{main_dir}_{name}".format(rule=xml_tags.TAG_RULE,
                                                   main_dir='_'.join(get_full_xml_tag(self.dirname)),
                                                   name=key.split('.')[0])
        if not name:
            self.update_files('migrate', content)
            self.update_files('upgrade', content)
        for x in name.split(','):
            self.update_files(x.strip(), content)

    def dummy_fnc(self, key, name):
        """Function is only dummy."""
        pass

    def update_text(self, key, name):
        """Function updates a text."""
        try:
            if key[name] is not None:
                # escape values so they can be loaded as XMLs
                escaped_text = html_escape_string(key[name])
                self.update_values_list(self.rule, "{"+name+"}", escaped_text)
        except KeyError:
            raise MissingTagsIniFileError

    def create_xml_from_ini(self, main):
        """
        The function creates group.xml file from INI file.
        All tag are replaced by function update_value_list

        Function also checks whether check script full fills all criteria
        """
        self.select_rules.append(xml_tags.SELECT_TAG)
        update_fnc = {
            'config_file': self.fnc_config_file,
            'check_script': self.fnc_check_script,
            'check_description': self.fnc_check_description,
            'solution': self.fnc_solution_text,
            'applies_to': self.dummy_fnc,
            'binary_req': self.dummy_fnc,
            'content_title': self.update_text,
            'content_description': self.update_text,
        }
        for key in self.keys:
            self.check_recommended_fields(key, script_name=main)
            # Add solution text into value
            if 'solution' in key:
                xml_tags.DIC_VALUES['solution_file'] = key['solution']
            else:
                xml_tags.DIC_VALUES['solution_file'] = 'solution.txt'

            # Add flag where will be shown content if in admin part or in user part
            if 'result_part' in key:
                xml_tags.DIC_VALUES['result_part'] = key['result_part']
            else:
                xml_tags.DIC_VALUES['result_part'] = 'admin'

            self.update_values_list(self.rule, "{rule_tag}",
                                    ''.join(xml_tags.RULE_SECTION))
            value_tag, check_export_tag = self.add_value_tag()
            self.update_values_list(self.rule, "{check_export}",
                                    ''.join(check_export_tag))
            self.update_values_list(self.rule, "{group_value}",
                                    ''.join(value_tag))

            try:
                for k, function in six.iteritems(update_fnc):
                    try:
                        function(key, k)
                    except IOError as e:
                        e_title = "Wrong value for tag '%s' in INI file '%s'\n" % (k, main)
                        e_msg = "'%s': %s" % (key[k], e.strerror)
                        print_error_msg(title=e_title, msg=e_msg)
                        raise MissingTagsIniFileError
            except MissingTagsIniFileError:
                title = "Following tag '%s' is missing in INI File %s\n" % (k, main)
                print_error_msg(title=title)
                raise MissingTagsIniFileError

            self.update_values_list(self.rule, '{group_title}', html_escape_string(key['content_title']))
            try:
                if 'mode' not in key:
                    self.fnc_update_mode(key['check_script'], 'migrate, upgrade')
                else:
                    self.fnc_update_mode(key['check_script'], key['mode'])
            except KeyError:
                pass



# # -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os
import unittest
import shutil
import stat
import tempfile

from xml.etree import ElementTree
try:
    from xml.etree.ElementTree import ParseError
except ImportError:
    from xml.parsers.expat import ExpatError as ParseError

from preuputils.compose import ComposeXML
from preup import xccdf
from preup import settings
from preuputils import variables
from preuputils.xml_utils import XmlUtils
from preuputils.oscap_group_xml import OscapGroupXml
from preup.utils import write_to_file, get_file_content
from preup.xml_manager import html_escape, html_escape_string

import base


class TestXMLCompose(base.TestCase):

    """Tests of right composing of contents in groups."""

    def setUp(self):
        dir_name = os.path.join(os.getcwd(), 'tests', 'FOOBAR6_7')
        self.result_dir = os.path.join(dir_name+variables.result_prefix)
        dir_name = os.path.join(dir_name, 'dummy')
        if os.path.exists(self.result_dir):
            shutil.rmtree(self.result_dir)
        shutil.copytree(dir_name, self.result_dir)
        template_file = ComposeXML.get_template_file()
        self.tree = None
        try:
            self.tree = ElementTree.parse(template_file).getroot()
        except IOError:
            assert False

        settings.autocomplete = False
        self.target_tree = ComposeXML.run_compose(self.tree, self.result_dir)
        self.assertTrue(self.target_tree)

    def tearDown(self):
        shutil.rmtree(self.result_dir)

    def test_compose(self):
        """Basic test of composing"""
        expected_groups = ['failed', 'fixed', 'needs_action',
                           'needs_inspection', 'not_applicable', 'pass',
                           'unicode']

        generated_group = []
        for group in self.target_tree.findall(xccdf.XMLNS + "Group"):
            generated_group.append(group.get('id'))
        self.assertEqual(['xccdf_preupg_group_'+x for x in expected_groups], generated_group)

    def test_unicode_xml(self):
        """
        Test processing of non-ascii characters inside title and description
        sections.
        """
        u_title = u'Čekujeme unicode u hasičů'
        u_descr = u'Hoří horní heršpická hospoda Hrbatý hrozen.'
        uni_xml = os.path.join(self.result_dir, "unicode", "group.xml")
        try:
            # XML files should be always in utf-8!
            lines = [x.decode('utf-8') for x in get_file_content(uni_xml, "rb", True, False)]
        except IOError:
            assert False
        title = filter(lambda x: u_title in x, lines)
        descr = filter(lambda x: u_descr in x, lines)
        self.assertTrue(title, "title is wrong ecoded or missing")
        self.assertTrue(descr, "description is wrong encoded or missing")

    def test_unicode_script_author(self):
        """Test processing of non-ascii characters for author section"""
        u_author = b'Petr Stod\xc5\xaflka'.decode(settings.defenc)
        script_file = os.path.join(self.result_dir, "unicode", "dummy_unicode.sh")
        settings.autocomplete = True
        self.target_tree = ComposeXML.run_compose(self.tree, self.result_dir)
        self.assertTrue(self.target_tree)
        try:
            lines = get_file_content(script_file, "rb", True)
        except IOError:
            assert False
        author = filter(lambda x: u_author in x, lines)
        self.assertTrue(author)


class TestXML(base.TestCase):

    """Main testing of right generating of XML files for OSCAP."""

    def setUp(self):
        self.dirname = os.path.join("tests", "FOOBAR6_7" + variables.result_prefix, "test")
        if os.path.exists(self.dirname):
            shutil.rmtree(self.dirname)
        os.makedirs(self.dirname)
        self.filename = os.path.join(self.dirname, 'test.ini')
        self.rule = []
        self.test_solution = "test_solution.txt"
        self.check_script = "check_script.sh"
        self.loaded_ini = {}
        test_ini = {'content_title': 'Testing content title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed'}
        self.loaded_ini[self.filename] = []
        self.loaded_ini[self.filename].append(test_ini)
        self.check_sh = """#!/bin/bash

#END GENERATED SECTION

#This is testing check script
 """
        check_name = os.path.join(self.dirname, self.check_script)
        write_to_file(check_name, "wb", self.check_sh)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)

        self.solution_text = """
A solution text for test suite"
"""
        test_solution_name = os.path.join(self.dirname, self.test_solution)
        write_to_file(test_solution_name, "wb", self.solution_text)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)
        self.xml_utils = XmlUtils(self.dirname, self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()

    def tearDown(self):
        shutil.rmtree(self.dirname)
        if os.path.exists(os.path.join(os.getcwd(), 'migrate')):
            os.unlink(os.path.join(os.getcwd(), 'migrate'))
        if os.path.exists(os.path.join(os.getcwd(), 'upgrade')):
            os.unlink(os.path.join(os.getcwd(), 'upgrade'))

    def test_group_xml(self):
        """Basic test for whole program"""
        self.assertTrue(self.loaded_ini[self.filename])
        self.assertTrue(self.rule)

    def test_xml_rule_id(self):
        rule_id = filter(lambda x: '<Rule id="xccdf_preupg_rule_test_check_script" selected="true">' in x, self.rule)
        self.assertTrue(rule_id)

    def test_xml_profile_id(self):
        profile = filter(lambda x: '<Profile id="xccdf_preupg_profile_default">' in x, self.rule)
        self.assertTrue(profile)

    def test_xml_rule_title(self):
        rule_title = filter(lambda x: "<title>Testing content title</title>" in x, self.rule)
        self.assertTrue(rule_title)

    def test_xml_config_file(self):
        conf_file = filter(lambda x: "<xhtml:li>/etc/named.conf</xhtml:li>" in x, self.rule)
        self.assertTrue(conf_file)

    def test_xml_fix_text(self):
        fix_text = filter(lambda x: "<fixtext>_test_SOLUTION_MSG_TEXT</fixtext>" in x, self.rule)
        self.assertTrue(fix_text)

    def test_xml_solution_type_text(self):
        self.loaded_ini[self.filename][0]['solution_type'] = "text"
        self.xml_utils = XmlUtils(self.dirname, self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        fix_text = filter(lambda x: "<fixtext>_test_SOLUTION_MSG_TEXT</fixtext>" in x, self.rule)
        self.assertTrue(fix_text)

    def test_xml_solution_type_html(self):
        self.loaded_ini[self.filename][0]['solution_type'] = "html"
        self.xml_utils = XmlUtils(self.dirname, self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        fix_text = filter(lambda x: "<fixtext>_test_SOLUTION_MSG_HTML</fixtext>" in x, self.rule)
        self.assertTrue(fix_text)

    def test_check_script_author(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = get_file_content(os.path.join(self.dirname, self.check_script), "rb", method=True)
        author = filter(lambda x: "test <test@redhat.com>" in x, lines)
        self.assertTrue(author)

    def test_xml_check_export_tmp_preupgrade(self):
        self.rule = self.xml_utils.prepare_sections()
        check_export = filter(lambda x: '<check-export export-name="TMP_PREUPGRADE" value-id="xccdf_preupg_value_test_check_script_state_tmp_preupgrade" />' in x, self.rule)
        self.assertTrue(check_export)

    def test_xml_current_directory(self):
        self.rule = self.xml_utils.prepare_sections()
        cur_directory = filter(lambda x: '<check-export export-name="CURRENT_DIRECTORY" value-id="xccdf_preupg_value_test_check_script_state_current_directory" />' in x, self.rule)
        self.assertTrue(cur_directory)

    def _create_temporary_dir(self):
        settings.UPGRADE_PATH = tempfile.mkdtemp()
        if os.path.exists(settings.UPGRADE_PATH):
            shutil.rmtree(settings.UPGRADE_PATH)
        os.makedirs(settings.UPGRADE_PATH)
        migrate = os.path.join(settings.UPGRADE_PATH, 'migrate')
        upgrade = os.path.join(settings.UPGRADE_PATH, 'upgrade')
        return migrate, upgrade

    def _delete_temporary_dir(self, migrate, upgrade):
        if os.path.exists(migrate):
            os.unlink(migrate)
        if os.path.exists(upgrade):
            os.unlink(upgrade)
        if os.path.exists(settings.UPGRADE_PATH):
            shutil.rmtree(settings.UPGRADE_PATH)

    def test_xml_migrate_not_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'migrate'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = []
        ini[self.filename].append(test_ini)
        xml_utils = XmlUtils(self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        try:
            upgrade_file = get_file_content(upgrade, 'rb', method=True)
        except IOError:
            upgrade_file = None
        self.assertIsNone(upgrade_file)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_upgrade_not_migrate(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'upgrade'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = []
        ini[self.filename].append(test_ini)
        xml_utils = XmlUtils(self.dirname, ini)
        xml_utils.prepare_sections()
        upgrade_file = get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        try:
            migrate_file = get_file_content(migrate, 'rb', method=True)
        except IOError:
            migrate_file = None
        self.assertIsNone(migrate_file)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_migrate_and_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'migrate, upgrade'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = []
        ini[self.filename].append(test_ini)
        xml_utils = XmlUtils(self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        upgrade_file = get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_not_migrate_not_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = []
        ini[self.filename].append(test_ini)
        xml_utils = XmlUtils(self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        upgrade_file = get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_check_script_reference(self):
        self.rule = self.xml_utils.prepare_sections()
        check_script_reference = filter(lambda x: '<check-content-ref href="check_script.sh" />' in x, self.rule)
        self.assertTrue(check_script_reference)

    def test_values_id(self):
        self.rule = self.xml_utils.prepare_sections()
        value_tmp_preupgrade = filter(lambda x: '<Value id="xccdf_preupg_value_test_check_script_state_tmp_preupgrade" operator="equals" type="string">' in x, self.rule)
        self.assertTrue(value_tmp_preupgrade)
        value_tmp_preupgrade_set = filter(lambda x: '<value>SCENARIO</value>' in x, self.rule)
        self.assertTrue(value_tmp_preupgrade_set)
        value_current_dir = filter(lambda x: '<Value id="xccdf_preupg_value_test_check_script_state_current_directory"' in x, self.rule)
        self.assertTrue(value_current_dir)
        value_current_dir_set = filter(lambda x: '<value>SCENARIO/test</value>' in x, self.rule)
        self.assertTrue(value_current_dir_set)

    def test_check_script_applies_to(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = get_file_content(os.path.join(self.dirname, self.check_script), "rb", method=True)
        applies = filter(lambda x: 'check_applies_to "test"' in x, lines)
        self.assertTrue(applies)

    def test_check_script_common(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = get_file_content(os.path.join(self.dirname, self.check_script), "rb", method=True)
        common = filter(lambda x: '. /usr/share/preupgrade/common.sh' in x, lines)
        self.assertTrue(common)

    def test_check_script_requires(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = get_file_content(os.path.join(self.dirname, self.check_script), "rb", method=True)
        check_rpm_to = filter(lambda x: 'check_rpm_to "bash" "sed"' in x, lines)
        self.assertTrue(check_rpm_to)


class TestIncorrectINI(base.TestCase):

    """
    Tests right processing of INI files including incorrect input which
    could make for crash with traceback.
    """

    def setUp(self):
        self.dir_name = "tests/FOOBAR6_7/incorrect_ini"
        os.makedirs(self.dir_name)
        self.filename = os.path.join(self.dir_name, 'test.ini')
        self.rule = []
        self.test_solution = "test_solution.sh"
        self.check_script = "check_script.sh"
        self.loaded_ini = {}
        self.loaded_ini[self.filename] = []
        self.test_ini = {'content_title': 'Testing content title',
                    'content_description': 'Some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'check_script': self.check_script,
                    'solution': self.test_solution,
                    'applies_to': 'test'}
        solution_text = """
A solution text for test suite"
"""
        check_sh = """#!/bin/bash

#END GENERATED SECTION

#This is testing check script
 """
        write_to_file(os.path.join(self.dir_name, self.test_solution), "wb", solution_text)
        write_to_file(os.path.join(self.dir_name, self.check_script), "wb", check_sh)

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_missing_tag_check_script(self):
        """Basic test for whole program"""
        self.test_ini.pop('check_script', None)
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))

    def test_missing_tag_solution_script(self):
        """Test of missing tag 'solution' - SystemExit should be raised"""
        self.test_ini.pop('solution', None)
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))

    def test_file_solution_not_exists(self):
        """Test of missing 'solution' file - SystemExit should be raised"""
        self.test_ini['solution'] = "this_should_be_unexpected_file.txt"
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))

    def test_file_check_script_not_exists(self):
        """Test of missing 'check_script' file"""
        self.test_ini['check_script'] = "this_should_be_unexpected_file.txt"
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))

    def test_check_script_is_directory(self):
        """
        Directory as input instead of regular file is incorrect input
        Tests issue #29
        """
        self.test_ini['check_script'] = '.'
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))

    def test_incorrect_tag(self):
        """
        Check occurrence of incorrect tag
        Tests issue #30
        """
        text_ini = '[preupgrade]\n'
        text_ini += '\n'.join([key + " = " + self.test_ini[key] for key in self.test_ini])
        text_ini += '\n[]\neliskk\n'
        write_to_file(self.filename, "wb", text_ini)
        oscap = OscapGroupXml(self.dir_name)
        self.assertRaises(SystemExit, oscap.find_all_ini)

    def test_secret_check_script(self):
        """Check occurrence of secret file for check script"""
        self.test_ini['check_script'] = '.minicheck'
        text = """#!/usr/bin/sh\necho 'ahojky'\n"""
        write_to_file(os.path.join(self.dir_name, self.check_script), "wb", text)
        self.loaded_ini[self.filename].append(self.test_ini)
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.assertRaises(SystemExit, lambda: list(self.xml_utils.prepare_sections()))


class TestGroupXML(base.TestCase):

    """Basic test for creating group.xml file"""

    #TODO: May should be deprecated and test could be moved under class TestXMLCompose

    def setUp(self):
        self.dir_name = "tests/FOOBAR6_7-results/test_group"
        os.makedirs(self.dir_name)
        self.filename = os.path.join(self.dir_name, 'group.ini')
        self.rule = []
        self.loaded_ini = {}
        test_ini = {'group_title': 'Testing content title'}
        self.assertTrue(test_ini)
        self.loaded_ini[self.filename] = []
        self.loaded_ini[self.filename].append(test_ini)

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_group_ini(self):
        """Basic test creation group.xml file"""
        self.xml_utils = XmlUtils(self.dir_name, self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        group_tag = filter(lambda x: '<Group id="xccdf_preupg_group_test_group" selected="true">' in x, self.rule)
        self.assertTrue(group_tag)
        title_tag = filter(lambda x: '<title>Testing content title</title>' in x, self.rule)
        self.assertTrue(title_tag)


class HTMLEscapeTest(base.TestCase):

    """Testing of right transform of unsafe characters to their entities"""

    def test_basic(self):
        """Basic test with quotation"""
        input_char = ['asd', '<qwe>', "this 'is' quoted"]
        output = html_escape(input_char)
        expected_output = ['asd', '&lt;qwe&gt;', 'this &apos;is&apos; quoted']
        self.assertEqual(output, expected_output)

    def test_basic_string(self):
        """Test if single string is escaped well"""
        input_char = "asd < > &"
        expected_output = "asd &lt; &gt; &amp;"
        output = html_escape_string(input_char)
        self.assertEqual(output, expected_output)

    def test_all(self):
        """Test if list of strings is escaped well"""
        tmp_input = ['a<', 'b>', "x'x", 'y"', 'z&z']
        output = html_escape(tmp_input)
        expected_ouput = ['a&lt;', 'b&gt;', 'x&apos;x', 'y&quot;', 'z&amp;z']
        self.assertEqual(output, expected_ouput)

    def test_amp_expand(self):
        """Test whether ampersand is not being expanded multiple times"""
        input_char = ['asd<>&']
        output = html_escape(input_char)
        expected_output = ['asd&lt;&gt;&amp;']
        self.assertEqual(output, expected_output)


class ComposeTest(base.TestCase):

    def setUp(self):
        pass

def suite():
    """Add classes which should be included in testing"""
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestGroupXML))
    suite.addTest(loader.loadTestsFromTestCase(TestXML))
    suite.addTest(loader.loadTestsFromTestCase(TestIncorrectINI))
    suite.addTest(loader.loadTestsFromTestCase(TestXMLCompose))
    suite.addTest(loader.loadTestsFromTestCase(HTMLEscapeTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



import unittest

"""
Use this module to subclass and enhance upstream classes
with test-suite-specific features like custom asserts
for unittest.TestCase.
"""


class TestCase(unittest.TestCase):

    # ported from 2.7's unittest
    def assertIsNone(self, obj, msg=None):
        """Same as self.assertTrue(obj is None), with a nicer
        default message."""
        if obj is not None:
            standardMsg = '%r is not None' % obj
            self.fail(self._formatMessage(msg, standardMsg))

    # ported from 2.7's unittest
    def assertIsNotNone(self, obj, msg=None):
        """Included for symmetry with assertIsNone."""
        if obj is None:
            standardMsg = 'unexpectedly None'
            self.fail(self._formatMessage(msg, standardMsg))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function
import os
import re
import rpm
import six
from preup.utils import get_file_content, write_to_file
from preup import settings


def html_escape_string(pattern):
    """
    escape single string for XML/HTML parsing
    """
    escapes = [
        ("&", "&amp;"),  # '&' has to be first!
        ("<", "&lt;"),
        (">", "&gt;"),
        ("'", "&apos;"),
        ('"', "&quot;"),
    ]
    result = pattern
    for esc, html_esc in escapes:
        result = result.replace(esc, html_esc)
    return result


def html_escape(text_list):
    """
    HTML escape text; text is a list of strings
    """
    for index, text in enumerate(text_list):
        text_list[index] = html_escape_string(text)

    return text_list


def link_update(value, extension, inplace):
    """
    Function replaces [link:sss] with either
    <a href="http:sss">sss</a> or
    <a hred="./sss">sss</a> for file link
    """
    prefix = ""
    postfix = " "
    if extension != "html":
        prefix = "html:"
        postfix = ' xmlns:html="http://www.w3.org/1999/xhtml/" '
    possible_links = ['http', 'https', 'ftp']
    if [x for x in possible_links if x in value]:
        return '<{1}a{2}href="{0}">{0}</{1}a>'.format(value.strip(),
                                                      prefix,
                                                      postfix)
    else:
        if inplace:
            return os.path.join('/root', settings.prefix, value.strip())
        if value.strip().startswith("/"):
            return ""
        else:
            return '<{1}a{2}href="./{0}">{0}</{1}a>'.format(value.strip(),
                                                            prefix,
                                                            postfix)


def bold_update(value, extension, inplace):
    """
    Function replaces [bold:sss] with <b>sss</b>
    """
    prefix = ""
    postfix = ""
    if extension != "html":
        prefix = "html:"
        postfix = ' xmlns:html="http://www.w3.org/1999/xhtml/" '
    return '<{1}b{2}>{0}</{1}b>'.format(value,
                                        prefix,
                                        postfix)


def tag_formating(text, extension):
    """
    Format tags like:
    [bold: some text] -> <b>some text</b>
    [link: http://127.0.0.1]
        ->
            <a href="http://127.0.0.1">http://127.0.0.1</a>
    [link: /var/cache/description.txt]
        ->
            <a href="/var/cache/description.txt">/var/cache/description.txt</a>
    """
    regular = r'\[(?P<tag>\w+):(?P<text>.+?)\]'
    update_dict = {
        'bold': bold_update,
        'link': link_update,
    }

    for index, line in enumerate(text):
        expr_re = re.compile(regular)
        string_match = re.findall(expr_re, line)
        if string_match:
            for match in string_match:
                # update = update_dict[string_match.group("tag")](match, extension)
                inplace = False
                if 'INPLACERISK:' in line:
                    inplace = True
                update = update_dict[match[0]](match[1], extension, inplace)
                if update != "":
                    line = re.sub(regular, update, line, count=1)
            text[index] = line
    return text


def remove_lines(string, regex_t, post_regex_t):
    """
    remove substring from string surrounded by regex
    Regexes are tuples: (regex, remove from start pos?), e.g.:
      ('<div id="main-table">', False)
    """
    s_re, remove_start = regex_t
    e_re, remove_end = post_regex_t

    s_search = re.search(s_re, string)
    e_search = re.search(e_re, string)
    if not s_search or not e_search:
        return string
    s_pos = s_search.start() if remove_start else s_search.end()
    e_pos = e_search.start() if remove_end else e_search.end()

    return string[:s_pos] + string[e_pos:]


def get_package_version(name):
    """
    Function return a package name and version
    """
    ts = rpm.TransactionSet()
    mi = ts.dbMatch()
    for h in mi:
        if h['name'] == name:
            return "%s-%s-%s" % (h['name'], h['version'], h['release'])


def add_preupg_scanner_info():
    """ add info about scanner to HTML report """
    template = """          <h2>Introduction</h2>
          <div>
            <h3>Scanner</h3>
            <ul>
%s            </ul>
          </div>
"""
    line = "              <li>%s</li>\n"
    packages = [
        'preupgrade-assistant',
        'preupgrade-assistant-contents',
        'preupgrade-assistant-contents-users',
    ]
    data = ''
    for package in packages:
        nvr = get_package_version(package)
        if nvr:
            l = line % nvr
            data += l
    return template % data


def clean_html(report_path):
    """
    Function cleans a report
    """
    file_content = get_file_content(report_path, 'rb')

    s_testres = ('[\t ]*<div id="intro">[\t ]*\n[\t ]*<h2>Introduction</h2>[\t ]*\n', False)
    e_testres = ('[\t ]*</table>[\t ]*\n[\t ]*</div>[\t ]*\n[\t ]*</div>[\t ]*\n', False)

    s_score = ('[\t ]*<div>[\t ]*\n[\t ]*<h3>Score</h3>\s*', True)
    e_score = ('[\t ]*</div>[\t ]*\n[\t ]*<div id="results-overview">[\t ]*\n', False)

    # remove test results
    nl = remove_lines(file_content, s_testres, e_testres)
    # remove score table
    #nl = remove_lines(nl, s_score, e_score)
    # sed XCCDF test results
    nl = re.sub('XCCDF test result', 'Preupgrade Assistant', nl)
    # add preupg nvr
    nl = re.sub('[\t ]*<h2>Introduction</h2>[\t ]*\n', add_preupg_scanner_info(), nl)

    write_to_file(report_path, 'wb', nl)


class XmlManager(object):
    """
    Class operates with XML oscap result
    """
    def __init__(self, dirname, scenario, filename, result_base):
        """
        dirname contains path to result
        """
        self.dirname = dirname
        self.filename = filename.split('.')[0]
        self.result_base = result_base
        self.scenario = scenario
        self.xml_solution_files = {}

    def get_updated_text(self, solution_text, text, line, extension):
        """Function updates a text in XML file"""
        updated_text = []
        if solution_text + "_TEXT" in line.strip():
            text = tag_formating(html_escape(text), extension)
            if extension == "html":
                new_line = "<br/>\n"
            else:
                new_line = "<html:br xmlns:html='http://www.w3.org/1999/xhtml/' />\n"
            updated_text = [x.strip() + new_line for x in text]
            if updated_text:
                updated_text = line.replace(solution_text + "_TEXT",
                                            ''.join(updated_text))
            else:
                updated_text = line.replace(solution_text + "_TEXT",
                                            "").replace(solution_text + "_HTML",
                                                        "")

        else:
            updated_text = text
        return updated_text

    def _return_correct_text_file(self, section, files):
        """
        Function returns only one text file
        based on section and list of txt files from content
        directory
        """
        found = False
        file_name = None
        for key, value in self.xml_solution_files.items():
            # section is in format _<path_content>
            section_name = section[1:] + "_"
            if section_name not in key:
                continue
            found = True
            # This will return only
            try:
                file_name = [txt for txt in files if txt == value][0]
                break
            except IndexError:
                pass
        return file_name

    def update_html(self, result_name, solution_files, extension="html"):
        """
         Function updates a XML or HTML file with relevant solution
         texts
        """
        orig_file = os.path.join(self.dirname,
                                 result_name + "." + extension)
        lines = get_file_content(orig_file, "rb", method=True)
        for dir_name, files in six.iteritems(solution_files):
            section = dir_name.replace(os.path.join(self.dirname, self.scenario),
                                       "").replace("/", "_")
            solution_text = section + "_SOLUTION_MSG"

            file_name = self._return_correct_text_file(section, files)
            if not file_name or file_name is None:
                continue
            else:
                text = get_file_content(os.path.join(dir_name, file_name),
                                        "rb",
                                        method=True)
            for cnt, line in enumerate(lines):
                # If in INPLACERISK: is a [link] then update them
                # to /root/pre{migrate,upgrade}/...
                if 'INPLACERISK:' in line.strip():
                    lines[cnt] = tag_formating([line], extension)[0]
                    continue
                # Find correct block
                if solution_text not in line.strip():
                    continue
                # Get updated text if it is HTML or TEXT
                lines[cnt] = self.get_updated_text(solution_text,
                                                   text,
                                                   line,
                                                   extension)

        if extension == 'xml':
            for cnt, line in enumerate(lines):
                if 'SOLUTION_MSG' in line.strip():
                    lines[cnt] = re.sub(r'>.*SOLUTION_MSG.*<', '><', line.strip())

        write_to_file(orig_file, "wb", lines)

    def find_solution_files(self, result_name, xml_solution_files):
        """
        Function finds all text files in conten
        and updates XML and HTML results
        """
        solution_files = {}
        self.xml_solution_files = xml_solution_files
        for dir_name, sub_dir, file_name in os.walk(self.dirname):
            files = [x for x in file_name if x.endswith(".txt")]
            if files:
                solution_files[dir_name] = files
        self.update_html(result_name, solution_files)
        self.update_html(result_name, solution_files, extension="xml")
        clean_html(os.path.join(self.dirname, result_name + ".html"))

    def remove_html_information(self):
        report_path = os.path.join(self.dirname, self.result_base + ".html")
        file_content = get_file_content(report_path, 'rb', method=True)
        detail_start = '<br /><br /><strong class="bold">Details:</strong><br />'
        detail_end = '[\t ]*<div class="xccdf-fixtext">'

        new_content = []
        found_section = False
        for line in file_content:
            if detail_start in line:
                found_section = True
                continue
            if detail_end in line:
                found_section = False
            if not found_section:
                new_content.append(line)

        write_to_file(report_path, 'wb', new_content)



temp_check_script = """#!/bin/bash

. /usr/share/preupgrade/common.sh

#END GENERATED SECTION

### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'.

"""

group_name = 'Specify a group name which content belongs to (like database):'
content_name = 'Specify a module name which will be created (like mysql):'
check_script = "Specify a script name which will be used for assessment: (check.sh)"
solution_text = "Specify a solution file which will be shown in report: (solution.txt)"
content_title = "Specify a content title:"
content_desc = "Would you like to specify a content description?"
content_desc_text = "Write down a content description:"
upgrade_path = "Specify a upgrade path (like RHEL6_7) where a content will be stored:"
summary_title = 'preupg-content-creator generates these files which should be updated:'
summary_directory = '- content was created in directory %s.'
summary_ini = '- ini file which defines a content is %s.'
summary_check = '- check script which does an assessment is %s. Update it before an usage'
summary_solution = '- solution text which informs about incompatilibies is %s. Update it before an usage'
check_path = "File %s already exists. Do you want to replace the file?"

text_for_testing = "\nFor testing content run these two commands:\n" \
                   "- preupg-create-group-xml %s to create XML file\n" \
                   "- preupg -c %s"


# -*- coding: utf-8 -*-

"""
Class creates a set of packages for migration scenario
"""

import os
from preup.utils import get_file_content


class YumGroupManager(object):
    """more intelligent dict; enables searching in yum groups"""
    def __init__(self):
        self.groups = {}

    def add(self, group):
        self.groups[group.name] = group

    def find_match(self, packages):
        """is there a group whose packages are subset of argument 'packages'?"""
        groups = []
        for group in self.groups.itervalues():
            if len(group.required) != 0:
                if group.match(packages):
                    groups.append(group)
        return groups

    def __str__(self):
        return "%s: %d groups" % (self.__class__.__name__, len(self.groups.values()))


class YumGroup(object):
    def __init__(self, name, mandatory, default, optional):
        self.name = name
        self.mandatory = mandatory
        self.mandatory_set = set(mandatory)
        self.default = default
        self.optional = optional
        self.required = set(mandatory + default)

    def __str__(self):
        return "%s (%d required packages)" % (self.name, len(self.required))

    def __repr__(self):
        return "<%s: M:%s D:%s O:%s>" % (self.name, self.mandatory, self.default, self.optional)

    def match(self, packages):
        return self.required.issubset(packages)

    def exclude_mandatory(self, packages):
        # New set with elements in packages but not in self.required
        return packages.difference(self.required)

    def exclude_optional(self, packages):
        # New set with elements in packages but not in self.optional
        return packages.difference(self.optional)


class YumGroupGenerator(object):
    """class for aggregating packages into yum groups"""

    def __init__(self, package_list, removed_packages, *args, **kwargs):
        """
        we dont take info about groups from yum, but from dark matrix, format is:

        group_name | mandatory packages | default packages | optional

        package_list is a list of packages which should aggregated into groups
        args is a list of filepaths to files where group definitions are stored
        """
        self.packages = set(package_list)
        self.removed_packages = set(removed_packages)
        self.gm = YumGroupManager()
        self.group_def_fp = []
        for p in args:
            if os.path.exists(p):
                self.group_def_fp.append(p)
                self._read_group_info()

    def _read_group_info(self):
        def get_packages(s):
            # get rid of empty strings
            return [x for x in s.strip().split(',') if x]

        for fp in self.group_def_fp:
            lines = get_file_content(fp, 'r', True)
            for line in lines:
                stuff = line.split('|')
                name = stuff[0].strip()
                mandatory = get_packages(stuff[1])
                default = get_packages(stuff[2])
                optional = get_packages(stuff[3])
                # why would we want empty groups?
                if mandatory or default or optional:
                    yg = YumGroup(name, mandatory, default, optional)
                    self.gm.add(yg)

    def remove_packages(self):
        for pkg in self.removed_packages:
            if pkg in self.packages:
                self.packages.remove(pkg)

    def get_list(self):
        groups = self.gm.find_match(self.packages)
        output_groups = []
        output_packages = self.packages
        for group in groups:
            if len(group.required) != 0:
                output_groups.append('@'+group.name)
                output_packages = group.exclude_mandatory(output_packages)
                output_packages = group.exclude_optional(output_packages)
        output_groups.sort()
        output_packages = list(output_packages)
        output_packages.sort()
        return output_groups, output_packages


class PackagesHandling(object):
    """class for replacing/updating package names"""

    def __init__(self, package_list, obsoleted, *args, **kwargs):
        """
        we dont take info about groups from yum, but from dark matrix, format is:

        group_name | mandatory packages | default packages | optional

        package_list is a list of packages which should aggregated into groups
        args is a list of filepaths to files where group definitions are stored
        """
        self.packages = package_list
        self.obsoleted = obsoleted

    def replace_obsolete(self):
        for pkg in self.obsoleted:
            fields = pkg.split()
            old_pkg = fields[0]
            new_pkg = fields[len(fields) - 1]
            found = [p for p in self.packages if p == old_pkg]
            if found:
                self.packages.append(new_pkg)

    def get_packages(self):
        return self.packages



# -*- coding: utf-8 -*-

"""
Class creates a kickstart for migration scenario
"""

import base64
import shutil

from pykickstart.constants import *
from pykickstart.parser import *
from pykickstart.version import *

from preup import utils
from preup.kickstart.kickstart_packages import YumGroupGenerator, PackagesHandling
from preup.kickstart.kickstart_partitioning import PartitionGenerator
from preup.logger import *
from preup.utils import write_to_file, get_file_content


class KickstartGenerator(object):
    """Generate kickstart using data from provided result"""

    def __init__(self, conf, dir_name, kick_start_name):
        self.dir_name = dir_name
        self.ks = None
        self.kick_start_name = kick_start_name
        self.ks_list = []
        self.repos = None
        self.users = None
        self.latest_tarball = ""
        self.temp_file = '/tmp/part-include'
        self.conf = conf
        self.groups = []
        self.packages = []
        self.part_layout = None

    def collect_data(self):
        self._remove_obsolete_data()
        collected_data = True
        self.ks = KickstartGenerator.load_or_default(KickstartGenerator.get_kickstart_path(self.dir_name))
        if self.ks is None:
            collected_data = False
        self.repos = KickstartGenerator.get_kickstart_repo('available-repos')
        self.users = KickstartGenerator.get_kickstart_users('Users')
        self.latest_tarball = self.get_latest_tarball()
        return collected_data

    def _remove_obsolete_data(self):
        if os.path.exists(KickstartGenerator.get_kickstart_path(self.dir_name)):
            lines = get_file_content(KickstartGenerator.get_kickstart_path(self.dir_name), "r", method=True)
            lines = [x for x in lines if not x.startswith('key')]
            write_to_file(KickstartGenerator.get_kickstart_path(self.dir_name), "w", lines)

    @staticmethod
    def get_kickstart_path(dir_name):
        return os.path.join(dir_name, 'anaconda-ks.cfg')

    @staticmethod
    def load_or_default(system_ks_path):
        """ load system ks or default ks """
        ksparser = KickstartParser(makeVersion())
        try:
            ksparser.readKickstart(system_ks_path)
        except (KickstartError, IOError):
            log_message("Can't read system kickstart at %s" % (system_ks_path))
            try:
                ksparser.readKickstart(os.path.join(KickstartGenerator.dir_name, settings.KS_TEMPLATE))
            except AttributeError:
                log_message("There is no KS_TEMPLATE_POSTSCRIPT specified in settings.py")
            except IOError, ioe:
                log_message("Can't read kickstart template %s %s" % (settings.KS_TEMPLATE, ioe))
                return None
        return ksparser

    @staticmethod
    def get_package_list(filename):
        """
        content packages/ReplacedPackages is taking care of packages, which were
        replaced/obsoleted/removed between releases. It produces a file with a list
        of packages which should be installed.
        """
        full_path_name = os.path.join(settings.KS_DIR, filename)
        if not os.path.exists(full_path_name):
            return []
        lines = get_file_content(full_path_name, 'rb', method=True, decode_flag=True)
        # Remove newline character from list
        lines = [line.strip() for line in lines]
        return lines

    @staticmethod
    def get_kickstart_repo(filename):
        """
        returns dictionary with names and URLs
        :param filename: filename with available-repos
        :return: dictionary with enabled repolist
        """
        try:
            lines = get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if not x.startswith('#') and not x.startswith(' ')]
        if not lines:
            return None
        repo_dict = {}
        for line in lines:
            fields = line.split('=')
            repo_dict[fields[0]] = fields[2]
        return repo_dict

    @staticmethod
    def get_kickstart_users(filename, splitter=":"):
        """
        returns dictionary with names and uid, gid, etc.
        :param filename: filename with Users in /root/preupgrade/kickstart directory
        :return: dictionary with users
        """
        try:
            lines = get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if not x.startswith('#') and not x.startswith(' ')]
        user_dict = {}
        for line in lines:
            fields = line.split(splitter)
            try:
                user_dict[fields[0]] = "%s:%s" % (fields[2], fields[3])
            except IndexError:
                pass
        return user_dict

    @staticmethod
    def _get_sizes(filename):
        part_sizes = {}
        lines = get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True, decode_flag=False)
        lines = [x for x in lines if x.startswith('/')]
        for line in lines:
            fields = line.strip().split(' ')
            part_name = fields[0]
            try:
                size = fields[2]
            except IndexError:
                size = fields[1]
            part_sizes[part_name] = size
        return part_sizes

    @staticmethod
    def get_installed_packages():
        prefix = 'RHRHEL7rpmlist_'
        result_list = []
        list_files = ['kept', 'optional', 'notbase',
                      'replaced', 'replaced-notbase']
        for l in list_files:
            try:
                result_list.extend(KickstartGenerator.get_package_list(prefix + l))
            except IOError:
                log_message("File '%s' was not found. Skipping package generation.", (prefix + l))
                return None
        return result_list

    def output_packages(self):
        """ outputs %packages section """
        installed_packages = KickstartGenerator.get_installed_packages()
        if installed_packages is None:
            return None
        try:
            obsoleted = KickstartGenerator.get_package_list('RHRHEL7rpmlist_obsoleted-required')
        except IOError:
            obsoleted = []
        ph = PackagesHandling(installed_packages, obsoleted)
        # remove files which are replaced by another package
        ph.replace_obsolete()

        if os.path.exists(os.path.join(settings.KS_DIR, 'RemovedPkg-optional')):
            try:
                removed_packages = KickstartGenerator.get_package_list('RemovedPkg-optional')
            except IOError:
                return None
        # TODO We should think about if ObsoletedPkg-{required,optional} should be used
        if not installed_packages or not removed_packages:
            return None
        abs_fps = [os.path.join(settings.KS_DIR, fp) for fp in settings.KS_FILES]
        ygg = YumGroupGenerator(ph.get_packages(), removed_packages, *abs_fps)
        self.groups, self.packages = ygg.get_list()
        ygg.remove_packages()

    def delete_obsolete_issues(self):
        """ Remove obsolete items which does not exist on RHEL-7 anymore"""
        self.ks.handler.bootloader.location = None

    def embed_script(self, tarball):
        tarball_content = get_file_content(tarball, 'rb', decode_flag=False)
        tarball_name = os.path.splitext(os.path.splitext(os.path.basename(tarball))[0])[0]
        script_str = ''
        try:
            script_path = settings.KS_TEMPLATE_POSTSCRIPT
        except AttributeError:
            log_message('KS_TEMPLATE_POSTSCRIPT is not defined in settings.py')
            return
        script_str = get_file_content(os.path.join(settings.KS_DIR, script_path), 'rb')
        if not script_str:
            log_message("Can't open script template: {0}".format(script_path))
            return

        script_str = script_str.replace('{tar_ball}', base64.b64encode(tarball_content))
        script_str = script_str.replace('{RESULT_NAME}', tarball_name)
        script_str = script_str.replace('{TEMPORARY_PREUPG_DIR}', '/var/tmp/preupgrade')
        script = Script(script_str, type=KS_SCRIPT_POST, inChroot=True)
        self.ks.handler.scripts.append(script)

    def save_kickstart(self):
        kickstart_data = self.ks.handler.__str__()
        write_to_file(self.kick_start_name, 'wb', kickstart_data)

    def update_kickstart(self, text, cnt):
        self.ks_list.insert(cnt, text)
        return cnt + 1

    @staticmethod
    def copy_kickstart_templates():
        # Copy kickstart files (/usr/share/preupgrade/kickstart) for kickstart generation
        for file_name in settings.KS_TEMPLATES:
            target_name = os.path.join(settings.KS_DIR, file_name)
            source_name = os.path.join(settings.source_dir, 'kickstart', file_name)
            if not os.path.exists(target_name) and os.path.exists(source_name):
                try:
                    shutil.copy(source_name, target_name)
                except IOError:
                    pass

    @staticmethod
    def get_volume_info(filename, first_index, second_index):
        try:
            volume_list = get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True, decode_flag=False)
        except IOError:
            log_message("File %s is missing. Partitioning layout has not to be complete." % filename, level=logging.WARNING)
            return None
        volume_info = {}
        for line in volume_list:
            fields = line.strip().split(':')
            volume_info[fields[first_index]] = fields[second_index]
        return volume_info

    def update_repositories(self, repositories):
        if repositories:
            for key, value in repositories.iteritems():
                self.ks.handler.repo.dataList().append(self.ks.handler.RepoData(name=key, baseurl=value.strip()))

    def update_users(self, users):
        if not users:
            return None
        for key, value in users.iteritems():
            uid, gid = value
            self.ks.handler.user.dataList().append(self.ks.handler.UserData(name=key, uid=int(uid), groups=[gid]))

    def get_partition_layout(self, lsblk, vgs, lvdisplay):
        """
        Returns dictionary with partition and realname and size
        :param filename:  filename with partition_layout in /root/preupgrade/kickstart directory
        :return: dictionary with layout
        """
        lsblk_filename = os.path.join(settings.KS_DIR, lsblk)
        try:
            layout = get_file_content(lsblk_filename, 'rb', method=True, decode_flag=False)
        except IOError:
            log_message("File %s was not generated by a content. Kickstart does not contain partitioning layout" % lsblk_filename)
            self.part_layout = None
            return None
        vg_info = []
        lv_info = []
        if vgs is not None:
            vg_info = KickstartGenerator.get_volume_info(vgs, 0, 5)
        if lvdisplay is not None:
            lv_info = KickstartGenerator.get_volume_info(lvdisplay, 0, 1)
        pg = PartitionGenerator(self.ks.handler, layout, vg_info, lv_info)
        pg.generate_partitioning()
        pg.get_partitioning()

    def get_prefix(self):
        return settings.tarball_prefix + settings.tarball_base

    def get_latest_tarball(self):
        tarball = None
        for directories, dummy_subdir, filenames in os.walk(settings.tarball_result_dir):
            preupg_files = [x for x in sorted(filenames) if x.startswith(self.get_prefix())]
            # We need a last file
            tarball = os.path.join(directories, preupg_files[-1])
        return tarball

    def filter_kickstart_users(self):
        kickstart_users = {}
        if not self.users:
            return None
        setup_passwd = KickstartGenerator.get_kickstart_users('setup_passwd')
        uidgid = KickstartGenerator.get_kickstart_users('uidgid', splitter='|')
        for user, ids in self.users.iteritems():
            if setup_passwd:
                if [x for x in setup_passwd.iterkeys() if user in x]:
                    continue
            if uidgid:
                if [x for x in uidgid.iterkeys() if user in x]:
                    continue
            kickstart_users[user] = ids.split(':')
        if not kickstart_users:
            return None
        return kickstart_users

    def comment_kickstart_issues(self):
        list_issues = ['user', 'repo', 'url', 'rootpw']
        kickstart_data = []
        try:
            kickstart_data = get_file_content(os.path.join(settings.KS_DIR, self.kick_start_name),
                                              'rb',
                                              method=True,
                                              decode_flag=False)
        except IOError:
            log_message("File %s is missing. Partitioning layout has not to be complete." % self.kick_start_name,
                        level=logging.WARNING)
            return None
        for index, row in enumerate(kickstart_data):
            tag = [com for com in list_issues if row.startswith(com)]
            if tag:
                kickstart_data[index] = "#" + row
        write_to_file(self.kick_start_name, 'wb', kickstart_data)

    def generate(self):
        if not self.collect_data():
            log_message("Important data are missing for kickstart generation.", level=logging.ERROR)
            return None
        self.output_packages()
        if self.packages or self.groups:
            self.ks.handler.packages.packageList = self.packages
            self.ks.handler.packages.groupList = self.groups
        self.ks.handler.packages.handleMissing = KS_MISSING_IGNORE
        self.ks.handler.keyboard.keyboard = 'us'
        self.update_repositories(self.repos)
        self.update_users(self.filter_kickstart_users())
        self.get_partition_layout('lsblk_list', 'vgs_list', 'lvdisplay')
        self.embed_script(self.latest_tarball)
        self.delete_obsolete_issues()
        self.save_kickstart()
        self.comment_kickstart_issues()
        return True

    def main(self):
        if not os.path.exists(os.path.join(settings.result_dir, settings.xml_result_name)):
            log_message("'preupg' command was not run yet. Run them before kickstart generation.")
            return 1

        KickstartGenerator.copy_kickstart_templates()
        dummy_ks = self.generate()
        if dummy_ks:
            log_message(settings.kickstart_text % settings.PREUPGRADE_KS)
        KickstartGenerator.kickstart_scripts()

    @staticmethod
    def kickstart_scripts():
        try:
            lines = utils.get_file_content(os.path.join(settings.common_dir,
                                                        settings.KS_SCRIPTS),
                                           "rb",
                                           method=True)
            for counter, line in enumerate(lines):
                line = line.strip()
                if line.startswith("#"):
                    continue
                if 'is not installed' in line:
                    continue
                cmd, name = line.split("=", 2)
                kickstart_file = os.path.join(settings.KS_DIR, name)
                utils.run_subprocess(cmd, output=kickstart_file, shell=True)
        except IOError:
            pass


def run():
    kg = KickstartGenerator()
    #print kg.generate()

    # group.packages() -> ['package', ...]
    #import ipdb ; ipdb.set_trace()
    return



#!/usr/bin/python2
# -*- coding: utf-8 -*-

import os
import distutils.command.sdist
import re
import subprocess
from distutils.command.install import INSTALL_SCHEMES
from setuptools import setup, find_packages
from preup.version import VERSION

project_name = "preupgrade-assistant"
project_url = "https://github.com/phracek/preupgrade-assistant/"
project_author = "Red Hat, Inc."
project_author_email = "phracek@redhat.com"
project_description = "Preupgrade assistant"
package_name = "%s" % project_name
package_module_name = project_name
package_version = VERSION

script_files = ['preupg', 'premigrate', 'preup_ui_manage',
                'preupg-xccdf-compose', 'preupg-create-group-xml',
                'preupg-content-creator', 'preupg-kickstart-generator']

data_files = {
    'preup_ui/report/fixtures/': ['preup_ui/report/fixtures/initial_data.json'],
    'preuputils/': ['preuputils/template.xml'],
    '/etc': ['preup-conf/preupgrade-assistant.conf'],
    '/usr/share/preupgrade/': ['common.sh', 'README', 'README.kickstart'],
    '/usr/share/preupgrade/common': ['common/scripts.txt', 'common/post_scripts.txt'],
    '/usr/share/preupgrade/kickstart': ['kickstart/default.ks', 'kickstart/finish.sh'],
    '/usr/share/preupgrade/postupgrade.d': ['postupgrade.d/copy_clean_conf.sh'],
}

# recursively add templates and static
paths = ['preup_ui/templates/', 'preup_ui/static/', 'preup_ui/lib/']
for path in paths:
    for root, dirs, files in os.walk(path):
        data_files[root] = [os.path.join(root, f) for f in files]

# override default tarball format with bzip2
distutils.command.sdist.sdist.default_format = {'posix': 'bztar'}

packages = find_packages(exclude=['tests'])

root_dir = os.path.dirname(__file__)
if root_dir != "":
    os.chdir(root_dir)

for scheme in INSTALL_SCHEMES.values():
    scheme["data"] = scheme["purelib"]

setup(
        name=package_name,
        version=package_version,
        url=project_url,
        author=project_author,
        author_email=project_author_email,
        description=project_description,
        packages=packages,
        data_files=data_files.items(),
        scripts=script_files,
        test_suite='tests.suite',
)



# -*- coding: utf-8 -*-
"""
The application module serves for creating a content
"""

from preup.logger import log_message, logging, set_level
from preup_creator.ui_helper import UIHelper


class Application(object):

    """Class for oscap binary and reporting results to UI"""

    def __init__(self, conf):
        """conf is preup.conf.Conf object, contains configuration"""
        self.conf = conf
        if self.conf.debug is None:
            set_level(logging.INFO)
        else:
            set_level(logging.DEBUG)
        self.ui_helper = UIHelper(self.conf.maindir)

    def run(self):
        self.ui_helper.take_manadatory_info()





"""
This class will ready the YAML file as INI file.
So no change is needed from maintainer point of view
"""

from __future__ import print_function, unicode_literals
import os
import sys
import six

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from preuputils.xml_utils import print_error_msg, XmlUtils
from preup.utils import write_to_file, check_file, get_file_content
from xml.etree import ElementTree
from preup import settings

try:
    from xml.etree.ElementTree import ParseError
except ImportError:
    from xml.parsers.expat import ExpatError as ParseError


class OscapGroupXml(object):

    """Class creates a XML file for OpenSCAP"""

    def __init__(self, dir_name):
        self.dirname = dir_name
        if dir_name.endswith('/'):
            self.main_dir = dir_name.split('/')[-3]
        else:
            self.main_dir = dir_name.split('/')[-2]
        self.lists = []
        self.loaded = {}
        self.filename = "group.xml"
        self.rule = []
        self.ret = {}

    def find_all_ini(self):
        """
        This function is used for finding all _fix files in the user defined
        directory
        """
        # solve python 2 & 3 compatibility
        if sys.version_info[0] == 2:
            config_load = lambda x, y: x.read(y)
            config_decode = lambda x: x.decode(settings.defenc)
        else:
            config_load = lambda x, y: x.read(y, settings.defenc)
            config_decode = lambda x: x

        for dir_name in os.listdir(self.dirname):
            if dir_name.endswith(".ini"):
                self.lists.append(os.path.join(self.dirname, dir_name))
        for file_name in self.lists:
            if check_file(file_name, "r") is False:
                continue
            try:
                config = configparser.ConfigParser()
                config_load(config, file_name)
                fields = {}
                if config.has_section('premigrate'):
                    section = 'premigrate'
                else:
                    section = 'preupgrade'
                for option in config.options(section):
                    fields[option] = config_decode(config.get(section, option))
                self.loaded[file_name] = [fields]
            except configparser.MissingSectionHeaderError:
                print_error_msg(title="Missing section header")
            except configparser.NoSectionError:
                print_error_msg(title="Missing section header")
            except configparser.ParsingError:
                print_error_msg(title="Incorrect INI file\n", msg=file_name)
                os.sys.exit(1)

    def collect_group_xmls(self):
        """The functions is used for collecting all INI files into the one."""
        # load content without decoding to unicode - ElementTree requests this
        try:
            self.ret[self.dirname] = (ElementTree.parse(os.path.join(self.dirname, "group.xml")).getroot())
        except ParseError as par_err:
            print("Encountered a parse error in file ", self.dirname, " details: ", par_err)
        return self.ret

    def write_xml(self):
        """The function is used for storing a group.xml file"""
        self.find_all_ini()
        self.write_list_rules()
        xml_utils = XmlUtils(self.dirname, self.loaded)
        self.rule = xml_utils.prepare_sections()
        file_name = os.path.join(self.dirname, "group.xml")
        try:
            write_to_file(file_name, "wb", ["%s" % item for item in self.rule])
        except IOError as ior:
            print ('Problem with write data to the file ', file_name, ior.message)

    def write_profile_xml(self, target_tree):
        """The function stores all-xccdf.xml file into content directory"""
        file_name = os.path.join(self.dirname, "all-xccdf.xml")
        print ('File which can be used by Preupgrade-Assistant is:\n', ''.join(file_name))
        try:
            # encoding must be set! otherwise ElementTree return non-ascii characters
            # as html entities instead, which are unsusable for us
            data = ElementTree.tostring(target_tree, "utf-8")
            write_to_file(file_name, "wb", data, False)
        except IOError as ioe:
            print ('Problem with writing to file ', file_name, ioe.message)

    def write_list_rules(self):
        rule_name = '_'.join(self.dirname.split('/')[1:])
        file_list_rules = os.path.join(settings.UPGRADE_PATH, settings.file_list_rules)
        lines = []
        if os.path.exists(file_list_rules):
            lines = get_file_content(file_list_rules, "rb", method=True)
        else:
            lines = []
        for values in six.itervalues(self.loaded):
            check_script = [v for k, v in six.iteritems(values[0]) if k == 'check_script']
            if check_script:
                check_script = os.path.splitext(''.join(check_script))[0]
                lines.append(settings.xccdf_tag + rule_name + '_' + check_script + '\n')
        write_to_file(file_list_rules, "wb", lines)





from __future__ import unicode_literals
import os
import shutil
import six
try:
    from hashlib import sha1
except ImportError:
    from sha import sha as sha1

from preup import settings
from preup.utils import get_interpreter, run_subprocess
from preup.utils import get_file_content, write_to_file
from preup.logger import log_message, logging


def get_all_postupgrade_files(dummy_verbose, dir_name):
    """Function gets all postupgrade files from dir_name"""
    postupg_scripts = []
    for root, dummy_sub_dirs, files in os.walk(dir_name):
        # find all files in this directory
        postupg_scripts.extend([os.path.join(root, x) for x in files])
    if not postupg_scripts:
        log_message("No postupgrade scripts available")
    return postupg_scripts


def get_hash_file(filename, hasher):
    """Function gets a hash from file"""
    content = get_file_content(filename, "rb", False, False)
    hasher.update(b'preupgrade-assistant' + content)
    return hasher.hexdigest()


def postupgrade_scripts(verbose, dirname):
    """
    The function runs postupgrade directory

    If dir does not exists the report and return
    """
    if not os.path.exists(dirname):
        log_message('There is no any %s directory' % settings.postupgrade_dir,
                    level=logging.WARNING)
        return

    postupg_scripts = get_all_postupgrade_files(verbose, dirname)
    if not postupg_scripts:
        return

    #max_length = max(list([len(x) for x in postupg_scripts]))

    log_message('Running postupgrade scripts:')
    for scr in sorted(postupg_scripts):
        interpreter = get_interpreter(scr, verbose=verbose)
        if interpreter is None:
            continue
        log_message('Executing script %s' % scr)
        cmd = "{0} {1}".format(interpreter, scr)
        run_subprocess(cmd, print_output=False, shell=True)
        log_message("Executing script %s ...done" % scr)


def get_hashes(filename):
    """Function gets all hashes from a filename"""
    if not os.path.exists(filename):
        return None
    hashed_file = get_file_content(filename, "rb").split()
    hashed_file = [x for x in hashed_file if "hashed_file" not in x]
    return hashed_file


def copy_modified_config_files(result_dir):
    """
    Function copies all modified files to dirtyconf directory.

    (files which are not mentioned in cleanconf directory)
    """
    etc_va_log = os.path.join(settings.cache_dir, settings.common_name, "rpm_etc_Va.log")
    try:
        lines = get_file_content(etc_va_log, "rb", method=True)
    except IOError:
        return
    dirty_conf = os.path.join(result_dir, settings.dirty_conf_dir)
    clean_conf = os.path.join(result_dir, settings.clean_conf_dir)
    for line in lines:
        try:
            (opts, flags, filename) = line.strip().split()
        except ValueError:
            return
        log_message("File name to copy '%s'" % filename,
                    print_output=0,
                    level=logging.INFO)
        new_filename = filename[1:]
        # Check whether config file exists in cleanconf directory
        file_name = os.path.join(clean_conf, new_filename)
        if os.path.exists(file_name):
            message = "Configuration file '%s' exists in '%s' directory"
            log_message(message % (file_name, clean_conf),
                        print_output=0,
                        level=logging.INFO)
            continue
        dirty_path = os.path.join(dirty_conf, os.path.dirname(new_filename))
        # Check whether dirtyconf directory with dirname(filename) exists
        if not os.path.exists(dirty_path):
            os.makedirs(dirty_path)
        # Copy filename to dirtyconf directory
        try:
            target_name = os.path.join(dirty_conf, new_filename)
            if os.path.islink(filename):
                filename = os.path.realpath(filename)
            if os.path.exists(target_name):
                log_message("File '%s' already exists in dirtyconf directory" % target_name,
                            print_output=0,
                            level=logging.INFO)
                continue
            shutil.copyfile(filename, target_name)
        except shutil.Error:
            log_message("Copying file '%s' to '%s' failed." % (filename, target_name),
                        print_output=0,
                        level=logging.INFO)
            continue
        except IOError:
            continue


def hash_postupgrade_file(verbose, dirname, check=False):
    """
    The function creates hash file over all scripts in postupgrade.d directory.

    In case of remediation it checks whether checksums are different and
    print what scripts were changed.
    """
    if not os.path.exists(dirname):
        message = 'Directory %s does not exist for creating checksum file'
        log_message(message, settings.postupgrade_dir, level=logging.ERROR)
        return

    postupg_scripts = get_all_postupgrade_files(verbose, dirname)
    if not postupg_scripts:
        return

    filename = settings.base_hashed_file
    if check:
        filename = settings.base_hashed_file + "_new"
    lines = []
    for post_name in postupg_scripts:
        lines.append(post_name + "=" + get_hash_file(post_name, sha1())+"\n")

    full_path_name = os.path.join(dirname, filename)
    write_to_file(full_path_name, "wb", lines)

    if check:
        hashed_file = get_hashes(os.path.join(dirname, settings.base_hashed_file))
        if hashed_file is None:
            message = 'Hashed_file is missing. Postupgrade scripts will not be executed'
            log_message(message, level=logging.WARNING)
            return False
        hashed_file_new = get_hashes(full_path_name)
        different_hashes = list(set(hashed_file).difference(set(hashed_file_new)))
        for file_name in [settings.base_hashed_file, filename]:
            os.remove(os.path.join(dirname, file_name))
        if different_hashes or len(different_hashes) > 0:
            message = 'Checksums are different in these postupgrade scripts: %s'
            log_message(message % different_hashes, level=logging.WARNING)
            return False
    return True


def special_postupgrade_scripts(result_dir):
    """
    The function copies a special postupgrade.d scripts.

    postupgrade_dict is a dictionary with old and new files
    Files are copied from
            /usr/share/preupgrade/postupgrade.d/<key> directory
    to
            /root/preupgrade/postupgrade.d/<val> directory
    with the corresponding names
    mentioned in postupgrade.d directory.
    """
    postupgrade_dict = {"copy_clean_conf.sh": "z_copy_clean_conf.sh"}

    for key, val in six.iteritems(postupgrade_dict):
        shutil.copy(os.path.join(settings.source_dir,
                                 settings.postupgrade_dir,
                                 key),
                    os.path.join(result_dir,
                                 settings.postupgrade_dir,
                                 val))




class MissingHeaderCheckScriptError(RuntimeError):
    """
    Error indicating problems with missing header in check script
    """

    pass


class MissingFileInContentError(RuntimeError):
    """
    Some file is missing in content
    """

    pass


class MissingTagsIniFileError(RuntimeError):
    """
    Some tags are missing in INI file
    """

    pass


class EmptyTagIniFileError(RuntimeError):
    """
    Some tag is empty. It is not approved
    """

    pass




from __future__ import unicode_literals, print_function
import unittest
import os

import base

from preup.kickstart import KickstartGenerator

PREUPGRADE_KS = 'preupgrade.ks'


def get_full_path(file_name):
    return os.path.join(os.getcwd(), 'tests', 'partition_data', file_name)


class TestPartitioning(base.TestCase):

    ks = None

    def setUp(self):
        kickstart_file = 'preupgrade.ks'
        self.ks = KickstartGenerator(os.path.join(os.getcwd(), 'tests', 'partition_data'), kickstart_file)

    def test_lvm_partitions(self):
        lvm_lsblk = get_full_path('lvm_lsblk_list')
        vgs_list = get_full_path('vgs_list')
        lvdisplay = get_full_path('lvdisplay')
        self.ks.get_partition_layout(lvm_lsblk, vgs_list, lvdisplay)
        expected_layout = ['clearpart --all',
                           'part /boot --size=500 --ondisk=vda',
                           'part pv.01 --size=9000',
                           'volgroup vg_rhel67 pv.01 --pesize=4096',
                           'logvol / --vgname=vg_rhel67 --size=8000 --name=lv_root',
                           'logvol swap --vgname=vg_rhel67 --size=1000 --name=lv_swap']
        self.assertEqual(expected_layout, self.ks.part_layout)

    def test_lvm_crypt_partitions(self):
        lvm_lsblk = get_full_path('lvm_crypt_lsblk_list')
        vgs_list = get_full_path('vgs_list')
        lvdisplay = get_full_path('lvdisplay')
        self.ks.get_partition_layout(lvm_lsblk, vgs_list, lvdisplay)
        expected_layout = ['clearpart --all',
                           'part /boot --size=500 --ondisk=vda',
                           'part pv.01 --size=9000 --encrypted',
                           'volgroup vg_rhel67 pv.01 --pesize=4096',
                           'logvol / --vgname=vg_rhel67 --size=8000 --name=lv_root',
                           'logvol swap --vgname=vg_rhel67 --size=1000 --name=lv_swap']
        self.assertEqual(expected_layout, self.ks.part_layout)

    def test_crypt_partitions(self):
        lvm_lsblk = get_full_path('crypt_lsblk_list')
        vgs_list = None
        lvdisplay = None
        self.ks.get_partition_layout(lvm_lsblk, vgs_list, lvdisplay)
        expected_layout = ['clearpart --all',
                           'part / --size=3000 --encrypted',
                           'part /boot --size=200 --ondisk=vda',
                           'part swap --size=2000 --ondisk=vda']
        self.assertEqual(expected_layout, self.ks.part_layout)

    def test_raid_crypt_partitions(self):
        raid_lsblk = get_full_path('raid_lsblk_list')
        self.ks.get_partition_layout(raid_lsblk, None, None)
        expected_layout = ['clearpart --all',
                           'part /boot --size=200 --ondisk=sda',
                           'part swap --size=1000 --ondisk=sda',
                           'part raid.01 --grow --size=2048',
                           'part raid.02 --grow --size=2048',
                           'raid / --level=1 --device=md1 raid.01 raid.02',
                           'part raid.03 --grow --size=2048',
                           'part raid.04 --grow --size=2048',
                           'raid /home --level=0 --device=md0 raid.03 raid.04 --encrypted --passphrase='
                           ]
        self.assertEqual(expected_layout, self.ks.part_layout)

    def test_raid_second_partitions(self):
        raid_lsblk = get_full_path('raid_lsblk_second_list')
        self.ks.get_partition_layout(raid_lsblk, None, None)
        expected_layout = ['clearpart --all',
                           'part /boot --size=1000 --ondisk=vda',
                           'part swap --size=1000 --ondisk=vdb',
                           'part raid.01 --grow --size=2048',
                           'part raid.02 --grow --size=2048',
                           'raid / --level=0 --device=md0 raid.01 raid.02',
                           ]
        self.assertListEqual(self.ks.part_layout, expected_layout)

    def test_native_partitioning(self):
        raid_lsblk = get_full_path('lsblk_native_list')
        self.ks.get_partition_layout(raid_lsblk, None, None)
        expected_layout = ['clearpart --all',
                           'part / --size=5000 --ondisk=vda',
                           'part /boot --size=200 --ondisk=vda',
                           'part /home --size=2000 --ondisk=vda',
                           'part swap --size=1000 --ondisk=vda',
                           ]
        self.assertListEqual(self.ks.part_layout, expected_layout)

    """def test_lvm_complicated_partitions(self):
        lvm_lsblk = get_full_path('lvm_complicated_lsblk_list')
        vgs_list = get_full_path('vgs_list_complicated')
        lvdisplay = get_full_path('lvdisplay_complicated')
        self.ks.get_partition_layout(lvm_lsblk, vgs_list, lvdisplay)
        expected_layout = ['clearpart --all',
                           'part --size=500M --ondisk=vda',
                           'part pv.1 --size 9.5G --encrypted',
                           'volgroup vg_rhel67 pv.01 --pesize=4096',
                           'logvol / --vgname=vg_rhel67 --size=8000 --name=lv_root',
                           'logvol swap --vgname=vg_rhel67 --size=1000 --name=lv_swap']
        self.assertEqual(expected_layout, self.ks.part_layout)
    """

    def tearDown(self):
        pass


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestPartitioning))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())




"""Configuration file, key names has to match values in cli.py"""

from __future__ import unicode_literals
import sys
import os


if os.path.basename(sys.argv[0]) == "premigrate":
    prefix = "premigrate"
else:
    prefix = "preupgrade"

defenc = "utf-8" if sys.getdefaultencoding() == "ascii" else sys.getdefaultencoding()

# dir where results of analysis are stored
result_dir = os.path.join("/root", prefix)

# Dir where tar balls are placed
tarball_result_dir = result_dir+"-results"

# base name of XML and HTML file with results
result_name = "result"

tarball_base = result_name + 's'
tarball_prefix = "preupg_"
tarball_name = tarball_prefix + tarball_base + "-{0}"

xml_result_name = result_name + '.xml'
html_result_name = result_name + '.html'

# base name of custom xsl stylesheet
xsl_sheet = "preup.xsl"

share_dir = "/usr/share"
# sources delivered by preupgrade assistant package
source_dir = os.path.join(share_dir, prefix)

# dir where the cached logs are stored
cache_dir = "/var/cache/preupgrade"

# file where the lock file stored
lock_file = "/var/run/preupgrade.pid"

# dir with log files"
log_dir = "/var/log/preupgrade"

# dir where the postupgrade scripts are placed
postupgrade_dir = "postupgrade.d"

# dir with preupgrade-scripts which are executed before reboot and upgrade.
preupgrade_name = "preupgrade-scripts"
preupgrade_scripts = os.path.join(result_dir, preupgrade_name)

# dirtyconfig directory used by preupgrade assistant
dirty_conf_dir = 'dirtyconf'

# cleanconfig directory used by preupgrade assistant
clean_conf_dir = 'cleanconf'

# cleanconf directory used by preupgrade assistant
# xccdf profile
profile = "xccdf_preupg_profile_default"

# name of dir with common files
common_name = "common"

# default directory is /var/tmp/preupgrade/common
# absolute path to dir with common files
common_dir = os.path.join(share_dir, prefix, common_name)

# path to file with definitions of common scripts
common_script = os.path.join(common_dir, "scripts.txt")

# Addons dir for 3rdparty contents
add_ons = "3rdparty"

# Default content file
content_file = "all-xccdf.xml"

# prefix of tag in fccdf files
xccdf_tag = "xccdf_preupg_rule_"

# name of the hash file
base_hashed_file = "hashed_file"

# name of the file which contains a list of rules
file_list_rules = "list_rules"

# path to file with definitions of common scripts
post_script = os.path.join(common_dir, "post_scripts.txt")

# kickstart and postupgrade.d directories
preupgrade_dirs = ['etc', dirty_conf_dir, clean_conf_dir,
                   'kickstart', postupgrade_dir, 'common',
                   'preupgrade-scripts', 'noauto_postupgrade.d']


readme_files = {'README': 'README',
                'README.kickstart': os.path.join('kickstart', 'README'),
                }

# Used for autogeneration check script issues
autocomplete = True

# new state needs_inspection
needs_inspection = "needs_inspection"

# new state needs_action
needs_action = "needs_action"

# The full license text
license = u"""Preupgrade assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
{0}
<new_line>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
<new_line>
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
<new_line>
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""

warning_text = "The Preupgrade Assistant is a diagnostics tool \n" \
               "and does not perform the actual upgrade.\n"
migration_text = "Running system is 32bit. Migration is possible only to 64bit system. See help --dst-arch option.\n"
assessment_text = "Assessment of the system, running checks / SCE scripts"
result_text = "Result table with checks and their results for '{0}':"
message = "We found some potential in-place upgrade risks.\n" \
          "Read the full report file '{0}' for more details."
converter_message = "At least one of these converters ({0}) needs to be installed."
kickstart_text = "The Preupgrade Assistant generates a kickstart file in '%s'.\n" \
                 "The Kickstart file contains:\n" \
                 "- users with UID/GID which you should create on Red Hat Enterprise Linux 7 system.\n" \
                 "- the partitioning layout which was used on this system\n" \
                 "- the package set which was installed on this system.\n" \
                 "The Kickstart file is pre-generated from this system and is not to be used directly for \n" \
                 "installation of Red Hat Enterprise Linux 7.\n" \
                 "The Kickstart file needs to be modified by administator.\n" \
                 "The directory contains:\n" \
                 "- cleanconf - configuration files which can be used on migrated system direcotry.\n" \
                 "- dirtyconf - configuration files which needs to be clarified by administrator.\n"

options_not_allowed = "Options --mode and --select-rules are not allowed together.\n"
list_rules = "List of all available rules:\n%s\n"
unknown_rules = "These rules does not exist:\n%s\n"
text_converters = {'w3m': '{0} -T text/html -dump {1} > {2}',
                   'lynx': '{0} -nonumbers -nolist -force_html -dump -nolist -width=255 {1} > {2}',
                   'elinks': '{0} --no-references -dump-width 255 --no-numbering -dump {1} > {2}',
                   }

ui_command = "preupg -u http://127.0.0.1:8099/submit/ -r {0}"
openssl_command = "openssl x509 -text -in {0} | grep -A1 1.3.6.1.4.1.2312.9.1"

UPGRADE_PATH = ""
KS_DIR = os.path.join(result_dir, 'kickstart')
KS_TEMPLATE = 'default.ks'
KS_TEMPLATE_POSTSCRIPT = 'finish.sh'
KS_TEMPLATES = [KS_TEMPLATE, KS_TEMPLATE_POSTSCRIPT]
KS_FILES = ['default_grouplist-el7', 'default-optional_grouplist-el7']
KS_SCRIPTS = "kickstart_scripts.txt"
PREUPGRADE_KS = os.path.join(result_dir, 'preupgrade.ks')
CPE_RHEL = 'redhat:enterprise_linux'
CPE_FEDORA = 'fedoraproject:fedora'
REPORTS = ['admin', 'user']
PREUPG_CONFIG_FILE = os.path.join('/etc', 'preupgrade-assistant.conf')

DEVEL_MODE = os.path.join(cache_dir, 'devel_mode')
